/******************************************************************************\
 * ODS2.H -- ODS-2.1 structure definitions                                    *
\******************************************************************************/

/* Home Block -- VBN 2 of INDEXF.SYS                                          */

struct HOME_BLOCK {
        unsigned long  HBLB;            /* Home Block LBN                     */
        unsigned long  AHLB;            /* Alternate Home Block LBN           */
        unsigned long  IHLB;            /* Backup Index File Header LBN       */
        unsigned short VLEV;            /* Structure Level and Version        */
        unsigned short SBCL;            /* Storage Bitmap Cluster Factor      */
        unsigned short HBVB;            /* Home Block VBN                     */
        unsigned short AHVB;            /* Backup Home Block VBN              */
        unsigned short IHVB;            /* Backup Index File Header VBN       */
        unsigned short IBVB;            /* Index File Bitmap VBN              */
        unsigned long  IBLB;            /* Index File Bitmap LBN              */
        unsigned long  FMAX;            /* Maximum Number of Files            */
        unsigned short IBSZ;            /* Index File Bitmap Size             */
        unsigned short RSVF;            /* Number of Reserved Files           */
        unsigned short DVTY;            /* Disk Device Type                   */
        unsigned short RVN;             /* Relative Volume Number             */
        unsigned short NVOL;            /* Number of Volumes                  */
        unsigned short VCHA;            /* Volume Characteristics             */
                                        /**************************************\
                                         * need to define                     *
                                         * CH.NDC, CH.NAT, CH.RCK, and CH.WCK *
                                        \**************************************/
        unsigned long  VOWN;            /* Volume Owner UIC                   */
        unsigned long  VSMX;            /* Volume Security Mask               */
        unsigned short VPRO;            /* Volume Protection                  */
        unsigned short DFPR;            /* Default File Protection            */
        unsigned short DRPC;            /* Default Record Protection          */
        unsigned short CHK1;            /* First Checksum                     */
        unsigned long  VDAT[2];         /* Volume Creation Date               */
        unsigned char  WISZ;            /* Default Window Size                */
        unsigned char  LRUC;            /* Directory Pre-Access Limit         */
        unsigned short FIEX;            /* Default File Extend                */
        unsigned char  H_1[388];        /* (not used)                         */
        unsigned char  SNAM[12];        /* Structure Name                     */
        unsigned char  INDN[12];        /* Volume Name                        */
        unsigned char  INDO[12];        /* Volume Owner                       */
        unsigned char  INDF[12];        /* Format Type                        */
        unsigned char  H_2[2];          /* (not used)                         */
        unsigned short CHK2;            /* Second Checksum                    */
};

/* File Header Header                                                         */
struct FILE_HEADER_HEADER {
        unsigned char  IDOF;            /* Ident Area Offset                  */
        unsigned char  MPOF;            /* Map Area Offset                    */
        unsigned char  ACOF;            /* Access Control List Offset         */
        unsigned char  RSOF;            /* Reserved Area Offset               */
        unsigned short FSEG;            /* Extension Segment Number           */
        unsigned short FLEV;            /* Structure Level and Version        */
        unsigned short FNUM;            /* File Number                        */
        unsigned short FSEQ;            /* File Sequence Number               */
        unsigned short FRVN;            /* Relative Volume Number             */
        unsigned short EFNU;            /* Extension File Number              */
        unsigned short EFSQ;            /* Extension File Sequence Number     */
        unsigned short ERVN;            /* Extension Relative Volume Number   */
        unsigned char  UFAT[32];        /* User File Attributes               */
        variant_union {
            unsigned long  FCHA;        /* File Characteristics               */
            variant_struct {
                unsigned char UCHA;     /* User Controlled Char.              */
                unsigned char SCHA;     /* System Controlled Char.            */
                unsigned char  F_1[2];  /* not named                          */
            } F_2;
        } F_3;
        unsigned char  F_4[2];          /* not used                           */
        unsigned char  USE;             /* Map Words in Use                   */
        unsigned char  PRIV;            /* Accessor Privilege Level           */
        variant_union {
            unsigned long  FOWN;        /* File Owner UIC                     */
            variant_struct {
                unsigned short PROG;    /* Programmer (Member) No.            */
                unsigned short PROJ;    /* Project (Group) Number             */
            } F_5;
        } F_6;
        unsigned short FPRO;            /* File Protection Code               */
                                        /**************************************\
                                         * Need to define                     *
                                         * FP.RDV, FP.WRV, FP.EXE, and FP.DEL *
                                        \**************************************/
        unsigned short RPRO;            /* Record Protection Code             */
                                        /**************************************\
                                         * Need to define                     *
                                         * RP.RDV, RP.WRV, RP.EXE, and RP.DEL *
                                        \**************************************/
        unsigned char  F_7[4];          /* Not Used                           */
        unsigned long  SEMK;            /* Security Mask                      */
};
#define HDHD sizeof(struct FILE_HEADER_HEADER)

/* File Header Ident Area                                                     */

struct FILE_HEADER_IDENT {
        unsigned char  FNAM[20];        /* File Name                          */
        unsigned short RVNO;            /* Revision Number                    */
        unsigned long  CRDT[2];         /* Creation Date and Time             */
        unsigned long  RVDT[2];         /* Revision Date and Time             */
        unsigned long  EXDT[2];         /* Expiration Date and Time           */
        unsigned long  BKDT[2];         /* Backup Date and Time               */
        unsigned char  ULAB[80];        /* User Label                         */
};
#define IDHD sizeof(struct FILE_HEADER_IDENT)

/* File Header                                                                */

struct FILE_HEADER {
        struct FILE_HEADER_HEADER H;    /* Header Area                        */
        char  FH_1[510-HDHD];           /* Ident, Map, ACL and Reserved Areas */
        unsigned short CKSM;            /* Block Checksum                     */
};

/* Retrieval pointers                                                         */

struct RTR_PTR_00 {
        unsigned PLACEMENT  : 14;       /* Placement Control                  */
        unsigned FORMAT     :  2;       /* Retrieval Pointer Format           */
};

struct RTR_PTR_01 {
        unsigned char   COUNT_01;       /* Count                              */
        unsigned HI_LBN_01  :  6;       /* High-Order Part of LBN             */
        unsigned FORMAT     :  2;       /* Retrieval Pointer Format           */
        unsigned short LO_LBN_01;       /* Low-Order Part of LBN              */
};

struct RTR_PTR_10 {
        unsigned COUNT_10   : 14;       /* Count                              */
        unsigned FORMAT     :  2;       /* Retrieval Pointer Format           */
        unsigned long     LBN_10;       /* Logical Block Number               */
};

struct RTR_PTR_11 {
        unsigned HI_CNT_11  : 14;       /* High-Order Part of Count           */
        unsigned FORMAT     :  2;       /* Retrieval Pointer Format           */
        unsigned short LO_CNT_11;       /* Low-Order Part of Count            */
        unsigned long     LBN_11;       /* Logical Block Number               */
};

